/*
 * Copyright (C) 1994, Silicon Graphics, Inc.
 * All Rights Reserved.
 *
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Silicon Graphics, Inc.;
 * the contents of this file may not be disclosed to third parties, copied or
 * duplicated in any form, in whole or in part, without the prior written
 * permission of Silicon Graphics, Inc.
 *
 * RESTRICTED RIGHTS LEGEND:
 * Use, duplication or disclosure by the Government is subject to restrictions
 * as set forth in subdivision (c)(1)(ii) of the Rights in Technical Data
 * and Computer Software clause at DFARS 252.227-7013, and/or in similar or
 * successor clauses in the FAR, DOD or NASA FAR Supplement. Unpublished -
 * rights reserved under the Copyright Laws of the United States.
 */
/*
|   Description:
|      A component class that creates a gizmo for editing 
|      orientation of normals, crease angle, and the 
|      minimum number of rows/columns in a noodle.
|
|   Author(s)          : Paul Isaacs
|
*/


#ifndef _NOODLE_SURFACE_GIZMO_
#define _NOODLE_SURFACE_GIZMO_

#include <Xm/Xm.h>
#include <Inventor/Xt/SoXtComponent.h>

extern class GeneralizedCylinder;
extern class NoodleSlider;

class NoodleSurfaceGizmo : public SoXtComponent {
  public:
    NoodleSurfaceGizmo( 
	Widget parent = NULL,
	const char *name = NULL, 
	SbBool buildInsideParent = TRUE);
    ~NoodleSurfaceGizmo();

     void setNoodle( GeneralizedCylinder *newNoodle );

  // internal:
      Widget buildWidget( Widget parent );

  private:
      Widget         _rowCol;
      Widget         flipNormButton;
      NoodleSlider  *creaseSlider;
      Widget         minNumRowsEdit;
      Widget         minNumColsEdit;

      GeneralizedCylinder *myNoodle;

      static void flipNormCallback( Widget, XtPointer, XtPointer);
      static void creaseCallback( void *, void *);
      static void minNumRowsCallback( Widget, XtPointer, XtPointer);
      static void minNumColsCallback( Widget, XtPointer, XtPointer);
};

#endif /* _NOODLE_SURFACE_GIZMO_ */
